

class Entity:

    # class attributes
    gameOver = False
    player = None
    entities_list = []

    def __init__(self, name, health):
        self.name = name
        self.maxHealth = health
        self.health = health
        Entity.entities_list.append(self)

    def __str__(self):
        return "<" + self.name + ">  " + \
               "HP:" + str(self.health) + "/" + str(self.maxHealth)

    def damage(self, amount):
        print(self.name + " has taken " + str(amount) + " of damage.")
        self.health = self.health - amount

        # if the entity died
        if self.health <= 0:
            print(self.name + " Died!!!")
            self.health = 0

            # remove it from the entity list
            if self in Entity.entities_list:
                Entity.entities_list.remove(self)

            # the game is over if the entities_list is empty
            Entity.gameOver = len(Entity.entities_list) == 0

    def play_round():
        pass

class Weapon:

    def __init__(self, name, power):
        self.name = name
        self.power = power
    
    def __str__(self):
        return "<" + self.name + ">  " + \
               "Power:" + str(self.power)

    def use_on(self, entity):
        entity.damage(self.power)


class Enemy(Entity):

    def __init__(self, name, health, target, weapon):
        Entity.__init__(self, name, health)
        self.target = target
        self.weapon = weapon
        self.round_number = 0

    def __str__(self):
        return Entity.__str__(self) + " Has a " + self.weapon.__str__()

    def play_round(self):
        print(self.name + " is using " + self.weapon.name + " against " + self.target.name + ".")
        self.weapon.use_on(self.target)

class Player(Entity):

    def __init__(self, name, health, weapon):
        Entity.__init__(self, name, health)
        self.weapon = weapon
        Entity.entities_list.remove(self)
        Entity.player = self
        self.defend = False

    def damage(self, amount):
        # if the player is in defend, do not take damage
        if(self.defend):
            print(self.name + " defends | " + str(amount) + " of damage was not taken.")
            return
        
        Entity.damage(self, amount) # use the method of the parent

        # if the player dies, is game over
        if self.health <= 0:
            Entity.gameOver = True

    def play_round(self):
        print("Select the Enemy to Attack")

        self.defend = False

        print("0) To Defend!!!")
        # print the list of all entities
        for i in range(0, len(Entity.entities_list)):
            print(str(i + 1) + ")" + str(Entity.entities_list[i]))

        index = int(input(">>>"))

        if(index == 0):
            print("Player is defending, +10 health")
            self.defend = True
            self.health += 10
            if self.health > self.maxHealth:
                self.health = self.maxHealth
            return

        enemy = Entity.entities_list[index - 1]

        print(self.name + " is using " + self.weapon.name + " against " + enemy.name)
        self.weapon.use_on(enemy)


def RunGame():

    round_number = 1
    while not Entity.gameOver:
        print("\nRound " + str(round_number))
        print(Entity.player)
        print("Players weapon is " + str(Entity.player.weapon))

        Entity.player.play_round()
        for e in Entity.entities_list:
            e.play_round()

        round_number += 1

    if Entity.player.health == 0:
        print("You died")
    else:
        print("You win")
